searchData={"items":[{"type":"module","title":"Ecto.ShortUUID","doc":"An Ecto type for Short UUID strings.","ref":"Ecto.ShortUUID.html"},{"type":"function","title":"Ecto.ShortUUID.autogenerate/0","doc":"Generates a random ShortUUID string for use in primary keys.\n\nThis is called by Ecto when a record is inserted if the schema has\n`@primary_key {:id, CustomShortUUIDType, autogenerate: true}`.","ref":"Ecto.ShortUUID.html#autogenerate/0"},{"type":"function","title":"Ecto.ShortUUID.cast/1","doc":"Casts a value to a ShortUUID.\n\nAccepts:\n- Standard UUID strings (converts them to ShortUUID format)\n- ShortUUID strings (validates them)\n\nReturns:\n- `{:ok, shortuuid}` on success\n- `:error` on failure","ref":"Ecto.ShortUUID.html#cast/1"},{"type":"function","title":"Ecto.ShortUUID.dump/1","doc":"Dumps a ShortUUID or standard UUID to binary format for database storage.\n\nAccepts:\n- ShortUUID strings\n- Standard UUID strings\n\nReturns:\n- `{:ok, binary_uuid}` on success\n- `:error` on failure","ref":"Ecto.ShortUUID.html#dump/1"},{"type":"function","title":"Ecto.ShortUUID.embed_as/1","doc":"Ecto v3 callback\nDictates how the type should be treated inside embeds.\nhttps://hexdocs.pm/ecto/Ecto.Type.html#c:embed_as/1","ref":"Ecto.ShortUUID.html#embed_as/1"},{"type":"function","title":"Ecto.ShortUUID.equal?/2","doc":"Ecto v3 callback\nChecks if two terms are semantically equal.\nhttps://hexdocs.pm/ecto/Ecto.Type.html#c:equal?/2","ref":"Ecto.ShortUUID.html#equal?/2"},{"type":"function","title":"Ecto.ShortUUID.generate/0","doc":"Generates a ShortUUID string based on a version 4 (random) UUID.","ref":"Ecto.ShortUUID.html#generate/0"},{"type":"function","title":"Ecto.ShortUUID.load/1","doc":"Loads a binary UUID from the database and converts it to ShortUUID format.\n\nReturns:\n- `{:ok, shortuuid}` on success\n- `:error` on failure\n\nRaises an `ArgumentError` if given a string UUID instead of binary.","ref":"Ecto.ShortUUID.html#load/1"},{"type":"type","title":"Ecto.ShortUUID.raw/0","doc":"A raw binary representation of a UUID.","ref":"Ecto.ShortUUID.html#t:raw/0"},{"type":"type","title":"Ecto.ShortUUID.shortuuid/0","doc":"A ShortUUID string.","ref":"Ecto.ShortUUID.html#t:shortuuid/0"},{"type":"function","title":"Ecto.ShortUUID.type/0","doc":"Returns the underlying schema type for the custom type.\n\nFor ShortUUID, this is always `:uuid` as the data is stored in UUID format\nin the database.","ref":"Ecto.ShortUUID.html#type/0"},{"type":"type","title":"Ecto.ShortUUID.uuid/0","doc":"A hex-encoded UUID string.","ref":"Ecto.ShortUUID.html#t:uuid/0"},{"type":"module","title":"Ecto.ShortUUID.Builder","doc":"A builder module for creating Ecto types that use custom ShortUUID implementations.","ref":"Ecto.ShortUUID.Builder.html"},{"type":"module","title":"Example - Ecto.ShortUUID.Builder","doc":"defmodule MyBase58UUID do\n      use ShortUUID.Builder, alphabet: :base58\n    end\n\n    defmodule MyShortUUID do\n      use Ecto.ShortUUID.Builder, encoder: MyBase58UUID\n    end\n\nThe specified encoder must be a [ShortUUID](https://github.com/gpedic/ex_shortuuid) compatible module.","ref":"Ecto.ShortUUID.Builder.html#module-example"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n**0.3.0**\n* Basic support for shortuuid v4, default alphabet only\n\n**0.2.0**\n* Add support for using shortuuid v3\n\n**0.1.5**\n* Update tests and test coverage\n* Use Ecto.UUID.load/1 directly and encode the resulting UUID instead of only accepting 128 bit binaries\n\n**0.1.4**\n* Fix a bug where 22 bits strings would fail to cast\n\n**0.1.3**\n\n* implement new Ecto.Type callbacks equal?/2 and embed_as/1\n\nImplement the new default callbacks to avoid compile warnings.\nImplementing them is preferable to using `use Ecto.Type` as it\ndoes not affect compatibility with Ecto v2\n\n**0.1.2**\n\n* update shortuuid depedency to ~> 2.1.1\n* this new version brings with it performance improvements\n\nshortuuid v2.1.0\n```","ref":"changelog.html"},{"type":"extras","title":"EctoShortUUIDBench - Changelog","doc":"benchmark name    iterations   average time\ncast/1 ShortUUID   100000000   0.03 µs/op\ncast/1 UUID           100000   17.02 µs/op\nload/1                500000   5.94 µs/op\ndump/1                100000   15.39 µs/op\ngenerate/0            100000   18.84 µs/op\n```\n\nshortuuid v2.1.1\n```","ref":"changelog.html#ectoshortuuidbench"},{"type":"extras","title":"EctoShortUUIDBench - Changelog","doc":"benchmark name    iterations   average time\ncast/1 ShortUUID   100000000   0.04 µs/op\ncast/1 UUID           500000   5.74 µs/op\nload/1                500000   3.65 µs/op\ndump/1                200000   8.78 µs/op\ngenerate/0            500000   7.53 µs/op\n```\n\n**0.1.1**\n\n* update shortuuid dep to v2.1.0 for binary encoding support\n* load/1 will now encode the binary directly without creating a string UUID first\n\n**0.1.0**\n\n* initial commit","ref":"changelog.html#ectoshortuuidbench"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2019 Goran Pedić\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"[![CI](https://github.com/gpedic/ecto_shortuuid/actions/workflows/ci.yml/badge.svg?branch=master)](https://github.com/gpedic/ecto_shortuuid/actions/workflows/ci.yml)\n[![Coverage Status](https://coveralls.io/repos/github/gpedic/ecto_shortuuid/badge.svg?branch=master)](https://coveralls.io/github/gpedic/ecto_shortuuid?branch=master)\n[![Hex.pm](https://img.shields.io/hexpm/v/ecto_shortuuid.svg)](https://hex.pm/packages/ecto_shortuuid)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecto_shortuuid/)\n[![License](https://img.shields.io/hexpm/l/ecto_shortuuid.svg)](https://github.com/gpedic/ecto_shortuuid/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/gpedic/ecto_shortuuid.svg)](https://github.com/gpedic/ecto_shortuuid/commits/master)\n\n# Ecto.ShortUUID\n\nEcto.ShortUUID is a custom [Ecto.Type](https://hexdocs.pm/ecto/Ecto.Type.html#content) which allows for Ecto to automatically encode UUIDs to [ShortUUIDs](https://github.com/gpedic/ex_shortuuid).\n\nShortUUID is binary-compatible with UUID, so it can be stored in a `uuid` column in a database.\n\nShortUUIDs can be used alongside `:binary_id` or as drop-in replacement for `:binary_id` primary and foreign keys, basically anywhere Ecto.UUID can be used.\n\nIf you're already using `:binary_id`(`Ecto.UUID`) for primary keys it is possible to simply switch from `:binary_id` to using `Ecto.ShortUUID` and vice versa, neither the underlying data nor DB schema need to be changed.\n\nFor example, we can see that calls to `Ecto.ShortUUID.dump/1` and `Ecto.UUID.dump/1` will  return the same binary in the following case:\n\n```elixir\n# let's get the encoded value\niex> Ecto.ShortUUID.cast(\"2a162ee5-02f4-4701-9e87-72762cbce5e2\")\n{:ok, \"9VprZJ9U7Tgg2PJ8BfTAek\"}\n\n# and show off that dump/1 works with both ShortUUIDs and UUIDs\niex> Ecto.ShortUUID.dump(\"9VprZJ9U7Tgg2PJ8BfTAek\")\n{:ok, <<42, 22, 46, 229, 2, 244, 71, 1, 158, 135, 114, 118, 44, 188, 229, 226>>}\n\n# dump/1 continues to work with regular UUIDs\niex> Ecto.ShortUUID.dump(\"2a162ee5-02f4-4701-9e87-72762cbce5e2\")\n{:ok, <<42, 22, 46, 229, 2, 244, 71, 1, 158, 135, 114, 118, 44, 188, 229, 226>>}\n\niex> Ecto.UUID.dump(\"2a162ee5-02f4-4701-9e87-72762cbce5e2\")\n{:ok, <<42, 22, 46, 229, 2, 244, 71, 1, 158, 135, 114, 118, 44, 188, 229, 226>>}\n\n# when a key is retrieved load/1 is called\n# with the binary representation of the UUID 2a162ee5-02f4-4701-9e87-72762cbce5e2\niex> Ecto.ShortUUID.load(<<42, 22, 46, 229, 2, 244, 71, 1, 158, 135, 114, 118, 44, 188, 229, 226>>)\n{:ok, \"9VprZJ9U7Tgg2PJ8BfTAek\"}\n\n# the same binary key continues to work with Ecto.UUID\niex> Ecto.UUID.load(<<42, 22, 46, 229, 2, 244, 71, 1, 158, 135, 114, 118, 44, 188, 229, 226>>)\n{:ok, \"2a162ee5-02f4-4701-9e87-72762cbce5e2\"}\n```\nWe can see that `Ecto.ShortUUID.dump/1` is backwards compatible and still accepts regular `UUIDs` and the value stored in the DB is exactly the same as when using the regular `:binary_id` type.","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `ecto_shortuuid` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_shortuuid, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Example project - Overview","doc":"You can check out the example project [ecto_shortuuid_example](https://github.com/gpedic/ecto_shortuuid_example) with defined config, schemas and seeds to quickly try out `ecto_shortuuid` for yourself.","ref":"readme.html#example-project"},{"type":"extras","title":"Usage - Overview","doc":"Using Ecto.ShortUUID is similar to using Ecto.UUID.","ref":"readme.html#usage"},{"type":"extras","title":"Schema - Overview","doc":"You can use the ShortUUID for a regular field\n\n```elixir\ndefmodule Post do\n  use Ecto.Schema\n\n  schema \"posts\" do\n    field :test, Ecto.ShortUUID\n  end\nend\n```\n\nTo use ShortUUIDs as primary keys we define the @primary_key attribute like this:\n\n```elixir\ndefmodule MyApp.User do\n  use Ecto.Schema\n\n  @primary_key {:id, Ecto.ShortUUID, autogenerate: true}\n\n  schema \"users\" do\n    field :name, :string\n    field :age, :integer, default: 0\n    has_many :posts, Post\n  end\nend\n```\n\nTo avoid having to define the primary key type in every schema in case you're using binary ids throughout you can make ShortUUIDs the default by defining the following in the underlying schema module.\n\n```elixir\n# Define a module to be used as base\ndefmodule MyApp.Schema do\n  defmacro __using__(_) do\n    quote do\n      use Ecto.Schema\n      @primary_key {:id, Ecto.ShortUUID, autogenerate: true}\n      @foreign_key_type Ecto.ShortUUID\n    end\n  end\nend\n\n# Now use MyApp.Schema to define new schemas\ndefmodule MyApp.User do\n  use MyApp.Schema\n\n  schema \"users\" do\n    field :name, :string\n    field :age, :integer, default: 0\n    has_many :posts, Post\n  end\nend\n```\nThis exactly the same as when using `:binary_id` as defined in the [Ecto docs - Schema attributes](https://hexdocs.pm/ecto/Ecto.Schema.html#module-schema-attributes).","ref":"readme.html#schema"},{"type":"extras","title":"Migration - Overview","doc":"The key has to be also defined as :uuid in the migration\n\n```elixir\ndefmodule MyApp.Repo.Migrations.CreateUser do\n  use Ecto.Migration\n\n  def change do\n    create table(:users, primary_key: false) do\n      add :id, :uuid, primary_key: true\n      add :name, :string\n      add :age, :integer\n      add :company_id, references(:companies, type: :uuid)\n\n      timestamps\n  end\nend\n```\n\nAlternatively if starting from scratch you can configure the default migration primary key type as:\n\n```elixir\nconfig :app, MyApp.Repo,\n  migration_primary_key: [name: :id, type: :binary_id]\n```\n\nRead more about config options in the [Ecto docs - Repo Configuration](https://hexdocs.pm/ecto_sql/Ecto.Migration.html#module-repo-configuration).","ref":"readme.html#migration"},{"type":"extras","title":"Custom Alphabets - Overview","doc":"Starting with version `0.4.0` Ecto.ShortUUID supports custom alphabets through two approaches.\n> Note: Custom alphabets are available in [ShortUUID](https://github.com/gpedic/ex_shortuuid) starting from version 4.0.0, and the ShortUUID.Behaviour is available since version 4.1.0.\n\nTo ensure you have the right version of ShortUUID for custom alphabets, you can explicitly specify it in your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:ecto_shortuuid, \"~> 0.4.0\"},\n    {:shortuuid, \"~> 4.1\"}  # Explicitly specify for custom alphabets/behaviors\n  ]\nend\n```","ref":"readme.html#custom-alphabets"},{"type":"extras","title":"Approach 1: Using ShortUUID.Builder - Overview","doc":"You can define a custom ShortUUID module with a specific alphabet using `ShortUUID.Builder`, then create a custom Ecto type with `Ecto.ShortUUID.Builder`:\n\n```elixir\n# Define a custom ShortUUID implementation with a specific alphabet\ndefmodule MyApp.Base58UUID do\n  use ShortUUID.Builder, alphabet: :base58\nend\n\n# Create a custom Ecto type that uses your ShortUUID implementation\ndefmodule MyApp.Base58EctoUUID do\n  use Ecto.ShortUUID.Builder, encoder: MyApp.Base58UUID\nend\n\n# Use in your schema\ndefmodule MyApp.User do\n  use Ecto.Schema\n  \n  @primary_key {:id, MyApp.Base58EctoUUID, autogenerate: true}\n  @foreign_key_type MyApp.Base58EctoUUID\n  \n  schema \"users\" do\n    # ...\n  end\nend\n```\n\n[ShortUUID](https://github.com/gpedic/ex_shortuuid) supports these predefined alphabets:\n- `:base57_shortuuid` - Default alphabet, omits ambiguous characters like \"l\", \"1\", \"I\", \"O\", \"0\"\n- `:base58` - Bitcoin alphabet\n- `:base62` - Alphanumeric alphabet\n- `:base32`, `:base32_hex`, `:base32_crockford`, `:base32_z` - Various base32 encodings\n- `:base64`, `:base64_url` - Base64 encodings\n\nYou can also define a custom alphabet string with at least 16 unique characters:\n\n```elixir\ndefmodule MyApp.CustomAlphabetUUID do\n  use ShortUUID.Builder, alphabet: \"0123456789ABCDEF-_+\"\nend\n```","ref":"readme.html#approach-1-using-shortuuid-builder"},{"type":"extras","title":"Approach 2: Implementing ShortUUID.Behaviour - Overview","doc":"For more control, you can implement the `ShortUUID.Behaviour` (available since [ShortUUID](https://github.com/gpedic/ex_shortuuid) v4.1.0) directly:\n\n```elixir\ndefmodule MyApp.CustomShortUUID do\n  @behaviour ShortUUID.Behaviour\n  \n  @impl true\n  def encode(uuid) do\n    # Your custom implementation\n    {:ok, \"custom-\" <> uuid}\n  end\n  \n  @impl true\n  def encode!(uuid) do\n    case encode(uuid) do\n      {:ok, encoded} -> encoded\n      {:error, reason} -> raise ArgumentError, message: reason\n    end\n  end\n  \n  @impl true\n  def decode(\"custom-\" <> uuid) do\n    # Your custom implementation\n    {:ok, uuid}\n  end\n  \n  @impl true\n  def decode!(encoded) do\n    case decode(encoded) do\n      {:ok, decoded} -> decoded\n      {:error, reason} -> raise ArgumentError, message: reason\n    end\n  end\nend\n\n# Create an Ecto type with your custom implementation\ndefmodule MyApp.CustomEctoUUID do\n  use Ecto.ShortUUID.Builder, encoder: MyApp.CustomShortUUID\nend\n```\n\nThis gives you complete control over encoding and decoding while maintaining compatibility with Ecto.","ref":"readme.html#approach-2-implementing-shortuuid-behaviour"},{"type":"extras","title":"Documentation - Overview","doc":"Additional documentation can\nbe found at [https://hexdocs.pm/ecto_shortuuid](https://hexdocs.pm/ecto_shortuuid).","ref":"readme.html#documentation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}